function sxt_composite,index,data,comp_index,unc=unc,                   $
	sx=sx,satpix=satpix,plot=plot,                                  $
	smooth_factor=smooth_factor,sfd=sfd,		        	$
	dc_interpolate=dc_interpolate,qtest=qtest,register=register,    $
	clean=clean,smoothpix=smoothpix,threshold=threshold
;+
; NAME:
;   sxt_composite
; PURPOSE:
;   Prepare a composite SXT image from 2 or 3 Half or Quarter Res images.
; CALLING SEQUENCE EXAMPLES:
;   cimg = sxt_composite(index,data,comp_index)
;   cimg = sxt_composite(index,data,[comp_index,unc=unc,sx=sx,           $
;        satpix=satpix,plot=plot,                                        $
;        smooth_factor=smooth_factor,sfd=sfd,                            $
;        dc_interpolate=dc_interpolate,qtest=qtest,register=register,    $
;        clean=clean,smoothpix=smoothpix,threshold])
; INPUTS:
;   index = SXT index structure
;   data  = 3-d data cube (at least 2 images)
;
;  If data is byte and no dc_data then		leak_sub
;  If data is byte and dc_data    then		decompress and data-dc_data
;  If data is not byte type       then		assume already decompressed, 
;						-- and background subtracted
; OUTPUT:
;  Function result is a 2-d long array (linear scaling) unless data is float
;  type, then output is a 2-d float array.
; OPTIONAL OUTPUT:
;    comp_index = index structure for composite image, made from index
;	of longest exposure with history records appended.
; OPTIONAL KEYWORD OUTPUT:
;   unc    = array of 1-sigma uncertainties in each pixel.  Use
;            sxt_dn_uncert for a temperature of 2.3 MK.
;   smoothpix = byte array of pixels containing boxcar-averaged data.
; OPTIONAL INPUT KEYWORDS:
;   dc_data = Dark frame data cube, use these rather than leak_sub.
;   sx	   = Indices of the data array. If not supplied, compute values
;	     that result in ==> sx = [short, medium, long exposure]
;   sd     = Indices of the background array.  If not supplied, 
;	     then must supply   sd = [short, medium, long exposure]
;   satpix = Saturated pixel array (required if data are not byte type).
;	     Must be 3-d and correspond to data(*,*,*).
;   plot   = If set, plot the results as we go.  The results are displayed
;	     on a logarithmic scale.
;   smooth_factor = Boxcar smooth the bleed pixels.  If not defined, the
;	     default is to smooth over 5 pixels.
;   sfd    = /sfd will cause the image to come back scaled as an sfd image
;	     (per second/HR pixel, and scaled by sfd_comp).
;   dc_interpolate = If set = 0 leak_sub will not use DC interpolation.
;		[Default is to use DC interpolation.]
;   register = If set, do a whole pixel registration of short exposures to 
;	     the long exposure
;   clean  = calls sxt_clean with defaults to despike output image.
;   threshold = if data are byte type this is the signal level in the
;	     shorter exposure below which the saturated regions of the
;            longer exposures are filled in with smoothed data for
;            statistical appearance sake.  (Default is 90 DN.)
; RESTRICTIONS:
;   Must be called separately for each composite output image.
; METHOD:
; ---------------------------------------------------------------------------
;1.  Replace the saturated pixels in the medium exposure with the
;    corresponding above-threshold time-normalized pixels from the 
;    short exposure.
;2.  Replace the saturated pixels in the medium exposure with smoothed
;    corresponding below-threshold time-normalized pixels from the
;    short exposure.
;
;    This is completes the process for the 2 image case.
;
;3.  For the 3 image case replace the saturated pixels in the long exposure
;    with the corresponding above-threshold time-normalized pixels from
;    step 2.
;4.  Replace saturated pixels in long exposure with smoothed 
;    corresponding below-threshold time-normalized pixels from step 2.
; ---------------------------------------------------------------------------
; MODIFICATION HISTORY:
;    8-Nov-91, L. W. Acton (Wrote composite)
;   19-May-92, LWA  (Several more mods in betwee)
;   13-may-93, J. R. Lemen, Re-wrote.  Calls sxt_satpix
;    1-Dec-93, MDM - Corrected error with the use of "s2" (when it is -1)
;		- Modification on selection of replacement pixels.
;		  It used to check (a) that it was high signal (ge 90)
;				   (b) that it was not saturated
;		  Added check so that it replaced pixels in the longer 
;		  exposure only when the longer exposure was saturated
;    15-Dec-93, SLF - Fixed typo (stapix->satpix)
;    22-feb-94, JRL - Corrected the algorithm; 
;		    - added smooth_factor, sfd, dc_interpolate, register keywords
;    17-Apr-94, LWA - Fixed /sfd scaling option (1000.*  was needed)
;		    - Changed /plot to plot raw input images.
;		    - Added RESTRICTIONS statement to header.
;		    - Added despike=despike keyword.
;    23-jun-94, JRL - Output is floating if input is a fltarr;
;		    - Call sfd_comp with index=index to normalize properly
;    08-May-95, LWA - Fixed to return negative values.
;    02-Nov-96, LWA - Added output of index with history record.
;		    - Made dc_data a keyword intput.
;		    - Made dc_interpolate the default is dc_data not input.
;    04-Nov-96, LWA - Removed forgotten diagnostic.
;    10-Jan-97, LWA - Corrected calling sequence examples.
;    20-Apr-99, LWA - Added /float and /orbit_correct to leak_sub call.
;                   - Version 3.01
;    14-Jul-99, BNH - history record wasn't being updated from LEAK_SUB.
;                   - Bumped version to 3.02.
;    11-Apr-00, LWA - Complete rewrite and improvement of logic.
;		    - Added unc, smoothpix and threshold keywords.
;		    - Changed version to 4.00.
;    11-Apr=00, lwa - Corrected typo and bumped version to 4.01.
;-

progverno=4.01*1000
nparams = n_params()
if nparams lt 2 then begin
   doc_library,'sxt_composite'
   return,'**  Must have at least 2 input parameters'
endif

;-------------------------
;  Check input parameters
;-------------------------

if NOT keyword_set(threshold) then threshold = 90 $
	else threshold = threshold
if NOT keyword_set(smooth_factor) and n_elements(smooth_factor) eq 0 $
	then smooth_factor = 5 $
	else smooth_factor = smooth_factor

sz0 = size(data)			; Data must be 3-d
if sz0(0) ne 3 then begin
   print,'***  Error in sxt_composite:  data must be 3-d'
   help,data
   return,-1
endif 

if n_elements(index) lt 2 then begin	; Must be at least 2 images
   print,'***  Error in sxt_composite:  index must have minimum length of 2'
   help,index
   return,-1
endif 

if n_elements(sx) eq 0 then sx = indgen(n_elements(index))
nimg = n_elements(sx)
sx = sx(sort(gt_expdur(index(sx))))	; short, medium, long

; Turn on history for this index record.
comp_index=index(sx(nimg-1))
his_index, /enable

; Make sure all the images have the same resolution:
if gt_res(index(sx(0))) ne total(gt_res(index(sx)))/ nimg then begin
    message,'The images must all have the same resolution'
    print,'Resolutions present = ',gt_res(index(sx),/str)
    return,-1
endif

; ----
; Run SXT_PREP to process the input data as required.
; ----
d_typ = sz0(sz0(0)+1)
if d_typ eq 1 then begin
   sxt_prep,index(sx),data(*,*,sx),iout,img,prep_unc,satpix,$
	/second_order_leak,/dc_interpolate,/dc_orbit_correct,$
	register=register,ref_image=sx(nimg-1),$
;       sxt_cleanx=clean,$    REMOVED PENDING REPAIR OF SXT_PREP  4/11/00
	/float
   t0 = gt_expdur(index(sx))
endif else begin
   sxt_prep,index(sx),data(*,*,sx),iout,img,register=register,$
;	sxt_cleanx=clean,$    REMOVED PENDING REPAIR OF SXT_PREP  4/11/00  
        ref_image=sx(nimg-1),/float
   t0 = gt_expdur(index(sx))	; t0 = [short, med, long]
endelse

; ----
;  Set up the plotting window and display the input images.
; ----
xsize = sz0(1)			;LWA  4/10/00
ysize = sz0(2)			;LWA  4/10/00
if keyword_set(plot) then begin
  set_plot,'x'			; Make sure
  wind_x = !d.x_size
  wind_y = !d.y_size
  if (wind_x lt 2*xsize) or (wind_y lt 2*ysize) then $
	wdef,lun,2*xsize,2*ysize,/free,/uright  else $
  	wshow & erase
  if (d_typ eq 1) then tvscl,data(*,*,sx(0)),0,ysize $
        else tvscl,alog(data(*,*,sx(0))>1),0,ysize
  if (d_typ eq 1) then tvscl,data(*,*,sx(1)),xsize,ysize $
        else tvscl,alog(data(*,*,sx(1))>1),xsize,ysize
  if (nimg eq 3) then begin
    if (d_typ eq 1) then tvscl,data(*,*,sx(2)),xsize,0 $
          else tvscl,alog(data(*,*,sx(2))>1),xsize,0
  endif
endif

; -----
; Set up the saturated pixel array
; -----
if n_elements(satpix) eq 0 then begin
   satpix = sxt_satpix(index,data,sx) 
   s_sat  = indgen(n_elements(sx))
endif else s_sat = sx

; -----
; Prepare the uncertainty array
; -----
if d_typ eq 1 then begin
   uncert = sxt_dn_unc(index(sx),img,te=alog10(2.3e6),/float,prep_unc)
endif else begin
   uncert = sxt_dn_unc(index(sx),img,te=alog10(2.3e6),/float)
endelse

; -----
; Prepare the composite images
; -----
if d_typ eq 1 then begin	;INPUT DATA ARE BYTE TYPE

   cimg0 = data(*,*,sx(0))
   cimg1 = data(*,*,sx(1))
   ii0 = where(cimg0 gt threshold and satpix(*,*,1) eq 1,nii0)
   cimgA = img(*,*,1)
   if nii0 gt 0 then cimgA(ii0) = (img(*,*,0))(ii0)*t0(1)/t0(0)
   ii00 = where(cimg0 le threshold and satpix(*,*,1) eq 1,nii00)
   if nii00 gt 0 then begin
      if smooth_factor gt 1 then begin
         cimgA(ii00) = (big_smooth(img(*,*,0),smooth_factor))(ii00)*t0(1)/t0(0)
      endif else begin
         cimgA(ii00) = (img(*,*,0))(ii00)*t0(1)/t0(0)
      endelse
   endif
   cimg = cimgA

   if nimg eq 3 then begin
      cimg2 = data(*,*,sx(2))
      ii1 = where(cimg1 gt threshold and satpix(*,*,2) eq 1,nii1)
      cimgB = img(*,*,2)
      if nii1 gt 0 then cimgB(ii1) = cimgA(ii1)*t0(2)/t0(1)
      ii10 = where(cimg1 le threshold and satpix(*,*,2) eq 1,nii10)
      if nii10 gt 0 then begin
         if smooth_factor gt 1 then begin
            cimgB(ii10) = (big_smooth(cimgA,smooth_factor))(ii10)*t0(2)/t0(1)
         endif else begin
            cimgB(ii10) = cimgA(ii10)*t0(2)/t0(1)
         endelse
      endif
      cimg = cimgB
   endif

endif else begin		;INPUT DATA HAVE BEEN DECOMPRESSED

   ii00 = where(satpix(*,*,1) eq 1,nii00)
   cimgA = img(*,*,1)
   if nii00 gt 0 then begin
      if smooth_factor gt 1 then begin
         cimgA(ii00) = (big_smooth(img(*,*,0),smooth_factor))(ii00)*t0(1)/t0(0)
      endif else begin
         cimgA(ii00) = (img(*,*,0))(ii00)*t0(1)/t0(0)
      endelse
   endif
   cimg = cimgA

   if nimg eq 3 then begin
      ii10 = where(satpix(*,*,2) eq 1,nii10)
      cimgB = img(*,*,2)
      if nii10 gt 0 then begin
         if smooth_factor gt 1 then begin
            cimgB(ii10) = (big_smooth(cimgA,smooth_factor))(ii10)*t0(2)/t0(1)
         endif else begin
            cimgB(ii10) = cimgA(ii10)*t0(2)/t0(1)
         endelse
      endif
      cimg = cimgB
   endif

endelse

; -----
; Prepare the uncertainty array.
; -----
if d_typ eq 1 then begin        ;INPUT DATA ARE BYTE TYPE

   if smooth_factor gt 1 then $
	mn_uncA = sqrt(big_smooth(uncert(*,*,0)^2.,smooth_factor))
   uncA = uncert(*,*,1)
   if nii0 gt 0 then uncA(ii0) = (uncert(*,*,0))(ii0)*t0(1)/t0(0)
   if nii00 gt 0 then begin
      if smooth_factor gt 1 then begin
         cimgA(ii00) = (mn_uncA)(ii00)*t0(1)/t0(0)
      endif else begin
         cimgA(ii00) = (uncert(*,*,0))(ii00)*t0(1)/t0(0)
      endelse
   endif
   unc = uncA

   if nimg eq 3 then begin
      if smooth_factor gt 1 then $
		mn_uncB = sqrt(big_smooth(uncA^2.,smooth_factor))
      uncB = uncert(*,*,2)
      if nii1 gt 0 then uncB(ii1) = uncA(ii1)*t0(2)/t0(1)
      if nii10 gt 0 then begin
         if smooth_factor gt 1 then begin
            uncB(ii10) = (mn_uncB)(ii10)*t0(2)/t0(1)
         endif else begin
            uncB(ii10) = uncA(ii10)*t0(2)/t0(1)
         endelse
      endif
      unc = uncB
   endif

endif else begin                ;INPUT DATA HAVE BEEN DECOMPRESSED

   if smooth_factor gt 1 then $
        mn_uncA = sqrt(big_smooth(uncert(*,*,0)^2.,smooth_factor))
   uncA = uncert(*,*,1)
   if nii00 gt 0 then begin
      if smooth_factor gt 1 then begin
         cimgA(ii00) = (mn_uncA)(ii00)*t0(1)/t0(0)
      endif else begin
         cimgA(ii00) = (uncert(*,*,0))(ii00)*t0(1)/t0(0)
      endelse
   endif
   unc = uncA

   if nimg eq 3 then begin
      if smooth_factor gt 1 then $
                mn_uncB = sqrt(big_smooth(uncA^2.,smooth_factor))
      uncB = uncert(*,*,2)
      if nii10 gt 0 then begin
         if smooth_factor gt 1 then begin
            uncB(ii10) = (mn_uncB)(ii10)*t0(2)/t0(1)
         endif else begin
            uncB(ii10) = uncA(ii10)*t0(2)/t0(1)
         endelse
      endif
      unc = uncB
   endif

endelse

; -----
; Prepare the array showing which pixels have been smoothed.
; -----
smoothpix = bytarr(sz0(1),sz0(2))
if nii00 gt 0 and smooth_factor gt 1 then smoothpix(ii00) = 1
if nii10 gt 0 and smooth_factor gt 1 then smoothpix(ii10) = 1

if keyword_set(plot) then tvscl,safe_log10(cimg)
if keyword_set(sfd) then cimg = sfd_comp(cimg,index=index(sx(nimg-1))) ; Scale as an sfd image 
if keyword_set(qtest) then stop 

;---------------------------------------------------------------------------
; This section builds the comp_index structure.
; ** Still need to include index.his.Q_COMPRESSION once I find out how
;	is it used.   LWA 2-Nov-96 **
;---------------------------------------------------------------------------
if nimg eq 2 then begin
   time_compos=[gt_time(index(sx(0))),gt_time(index(sx(1))),0]
   day_compos=[gt_day(index(sx(0))),gt_day(index(sx(1))),0]
endif else begin
   time_compos=[gt_time(index(sx(0))),gt_time(index(sx(1))),$
	gt_time(index(sx(2)))]
   day_compos=[gt_day(index(sx(0))),gt_day(index(sx(1))),gt_day(index(sx(2)))]
endelse

his_index, comp_index, 0, 'q_composite', progverno*1000
his_index, comp_index, 0, 'time_compos', time_compos
his_index, comp_index, 0, 'day_compos', day_compos

return,cimg 
end

